@@ -39,6 +39,7 @@ struct _GrApp
 {
         GtkApplication parent_instance;
 
+        SoupSession *session;
         GrShellSearchProvider *search_provider;
         GtkCssProvider *css_provider;
 };
@@ -51,6 +52,7 @@ gr_app_finalize (GObject *object)
 {
         GrApp *self = GR_APP (object);
 
+        g_clear_object (&self->session);
         g_clear_object (&self->search_provider);
         g_clear_object (&self->css_provider);
 
@@ -205,6 +207,20 @@ report_issue_activated (GSimpleAction *action,
         gr_window_show_report_issue (GR_WINDOW (win));
 }
 
+static void
+help_activated (GSimpleAction *action,
+                GVariant      *parameter,
+                gpointer       app)
+{
+        GList uris = { (gpointer)"help:org.gnome.Recipes", NULL, NULL };
+        g_autoptr(GAppInfo) info = NULL;
+        g_autoptr(GError) error = NULL;
+
+        info = g_app_info_get_default_for_uri_scheme ("help");
+        if (!g_app_info_launch_uris (info, &uris, NULL, &error))
+                g_warning ("Failed to launch %s: %s", g_app_info_get_name (info), error->message);
+}
+
 static void
 verbose_logging_activated (GSimpleAction *action,
                            GVariant      *parameter,
@@ -228,6 +244,7 @@ static GActionEntry app_entries[] =
         { "search", search_activated, "s", NULL, NULL },
         { "quit", quit_activated, NULL, NULL, NULL },
         { "report-issue", report_issue_activated, NULL, NULL, NULL },
+        { "help", help_activated, NULL, NULL, NULL },
         { "verbose-logging", verbose_logging_activated, "b", NULL, NULL }
 };
 
@@ -382,6 +399,10 @@ gr_app_init (GrApp *self)
                                        _("Turn on verbose logging"), NULL);
 
         g_log_set_writer_func (gr_log_writer, NULL, NULL);
+
+        self->session = soup_session_new_with_options (SOUP_SESSION_USER_AGENT,
+                                                       PACKAGE_NAME "/" PACKAGE_VERSION,
+                                                       NULL);
 }
 
 static int
@@ -435,3 +456,9 @@ gr_app_new (void)
                              "flags", G_APPLICATION_HANDLES_OPEN | G_APPLICATION_CAN_OVERRIDE_APP_ID,
                              NULL);
 }
+
+SoupSession *
+gr_app_get_soup_session (GrApp *app)
+{
+        return app->session;
+}
