@@ -26,6 +26,7 @@
 
 #include "gr-cooking-page.h"
 #include "gr-cooking-view.h"
+#include "gr-settings.h"
 #include "gr-utils.h"
 #include "gr-window.h"
 #include "gr-recipe.h"
@@ -95,14 +96,14 @@ gr_cooking_page_init (GrCookingPage *self)
 static guint
 get_cooking_overlay_count (void)
 {
-        g_autoptr(GSettings) settings = g_settings_new ("org.gnome.Recipes");
+        GSettings *settings = gr_settings_get ();
         return g_settings_get_uint (settings, "cooking");
 }
 
 static void
 set_cooking_overlay_count (uint count)
 {
-        g_autoptr(GSettings) settings = g_settings_new ("org.gnome.Recipes");
+        GSettings *settings = gr_settings_get ();
         g_settings_set_uint (settings, "cooking", count);
 }
 
@@ -529,7 +530,7 @@ void
 gr_cooking_page_set_recipe (GrCookingPage *page,
                             GrRecipe      *recipe)
 {
-        g_autoptr(GArray) images = NULL;
+        g_autoptr(GPtrArray) images = NULL;
         const char *id;
         const char *instructions;
 
