@@ -38,7 +38,10 @@ struct _GrCuisinesPage
 {
         GtkBox parent_instance;
 
-        GtkWidget *top_box;
+        GtkWidget *cuisines_box;
+        GtkWidget *cuisines_box2;
+        GtkWidget *cuisines_more;
+        GtkWidget *cuisines_expander_image;
         GtkWidget *seasonal_box;
         GtkWidget *seasonal_box2;
         GtkWidget *seasonal_more;
@@ -59,6 +62,17 @@ cuisines_page_finalize (GObject *object)
         G_OBJECT_CLASS (gr_cuisines_page_parent_class)->finalize (object);
 }
 
+static void
+set_cuisines_expanded (GrCuisinesPage *page,
+                       gboolean        expanded)
+{
+        gtk_revealer_set_transition_duration (GTK_REVEALER (page->cuisines_more), expanded ? 250 : 0);
+
+        gtk_revealer_set_reveal_child (GTK_REVEALER (page->cuisines_more), expanded);
+        gtk_image_set_from_icon_name (GTK_IMAGE (page->cuisines_expander_image),
+                                      expanded ? "pan-up-symbolic" : "pan-down-symbolic", 1);
+}
+
 static void
 set_seasonal_expanded (GrCuisinesPage *page,
                        gboolean        expanded)
@@ -78,14 +92,26 @@ gr_cuisines_page_unexpand (GrCuisinesPage *page)
         transition = gtk_revealer_get_transition_type (GTK_REVEALER (page->seasonal_more));
         gtk_revealer_set_transition_type (GTK_REVEALER (page->seasonal_more),
                                           GTK_REVEALER_TRANSITION_TYPE_NONE);
-
         set_seasonal_expanded (page, FALSE);
-
         gtk_revealer_set_transition_type (GTK_REVEALER (page->seasonal_more), transition);
+
+        transition = gtk_revealer_get_transition_type (GTK_REVEALER (page->cuisines_more));
+        gtk_revealer_set_transition_type (GTK_REVEALER (page->cuisines_more),
+                                          GTK_REVEALER_TRANSITION_TYPE_NONE);
+        set_cuisines_expanded (page, FALSE);
+        gtk_revealer_set_transition_type (GTK_REVEALER (page->cuisines_more), transition);
 }
 
 static void
-expander_button_clicked (GrCuisinesPage *page)
+cuisines_expander_button_clicked (GrCuisinesPage *page)
+{
+        gboolean expanded;
+
+        expanded = gtk_revealer_get_reveal_child (GTK_REVEALER (page->cuisines_more));
+        set_cuisines_expanded (page, !expanded);
+}
+static void
+season_expander_button_clicked (GrCuisinesPage *page)
 {
         gboolean expanded;
 
@@ -93,6 +119,22 @@ expander_button_clicked (GrCuisinesPage *page)
         set_seasonal_expanded (page, !expanded);
 }
 
+static void
+cuisine_clicked (GrCategoryTile *tile,
+                 GrCuisinesPage *page)
+{
+        GtkWidget *window;
+        const char *cuisine;
+        const char *title;
+
+        window = gtk_widget_get_ancestor (GTK_WIDGET (tile), GR_TYPE_WINDOW);
+
+        cuisine = gr_category_tile_get_category (tile);
+        title = gr_category_tile_get_label (tile);
+
+        gr_window_show_cuisine (GR_WINDOW (window), cuisine, title);
+}
+
 static void
 populate_cuisines (GrCuisinesPage *page)
 {
@@ -103,8 +145,14 @@ populate_cuisines (GrCuisinesPage *page)
         int i, j;
         GrRecipeStore *store;
         int tiles;
+        g_autoptr(GHashTable) seen = NULL;
+        g_autofree char **names = NULL;
+        guint len;
 
-        container_remove_all (GTK_CONTAINER (page->top_box));
+        container_remove_all (GTK_CONTAINER (page->cuisines_box));
+        container_remove_all (GTK_CONTAINER (page->cuisines_box2));
+
+        seen = g_hash_table_new (g_str_hash, g_str_equal);
 
         store = gr_recipe_store_get ();
 
@@ -130,19 +178,52 @@ populate_cuisines (GrCuisinesPage *page)
         tile = gr_cuisine_tile_new (page->featured, TRUE);
         gtk_widget_show (tile);
         gtk_widget_set_halign (tile, GTK_ALIGN_FILL);
-        gtk_grid_attach (GTK_GRID (page->top_box), tile, 0, 0, 2, 1);
+        gtk_grid_attach (GTK_GRID (page->cuisines_box), tile, 0, 0, 2, 1);
+        g_hash_table_add (seen, page->featured);
 
         tiles = 0;
         for (i = 0; i < length; i++) {
                 if (strcmp (page->featured, cuisines[i]) == 0)
                         continue;
 
-                tile = gr_cuisine_tile_new (cuisines[i], FALSE);
-                gtk_widget_show (tile);
-                gtk_widget_set_halign (tile, GTK_ALIGN_FILL);
-                gtk_grid_attach (GTK_GRID (page->top_box), tile, tiles % 2, 1 + tiles / 2, 1, 1);
+                if (tiles < 4) {
+                        tile = gr_cuisine_tile_new (cuisines[i], FALSE);
+                        gtk_widget_show (tile);
+                        gtk_widget_set_halign (tile, GTK_ALIGN_FILL);
+                        gtk_grid_attach (GTK_GRID (page->cuisines_box), tile, tiles % 2, 1 + tiles / 2, 1, 1);
+                }
+                else {
+                        const char *title;
 
+                        gr_cuisine_get_data (cuisines[i], &title, NULL, NULL);
+                        tile = gr_category_tile_new_with_label (cuisines[i], title);
+                        gtk_widget_show (tile);
+                        g_signal_connect (tile, "clicked", G_CALLBACK (cuisine_clicked), page);
+                        gtk_container_add (GTK_CONTAINER (page->cuisines_box2), tile);
+                }
                 tiles++;
+
+                g_hash_table_add (seen, cuisines[i]);
+        }
+
+        names = gr_recipe_store_get_all_cuisines (store, &len);
+        for (i = 0; i < len; i++) {
+               g_autofree char *tmp = NULL;
+               const char *title;
+
+                tmp = g_strstrip (g_strdup (names[i]));
+                if (strcmp (tmp, "") == 0)
+                        continue;
+
+                if (g_hash_table_contains (seen, tmp))
+                        continue;
+
+                gr_cuisine_get_data (tmp, &title, NULL, NULL);
+
+                tile = gr_category_tile_new_with_label (tmp, title);
+                gtk_widget_show (tile);
+                g_signal_connect (tile, "clicked", G_CALLBACK (cuisine_clicked), page);
+                gtk_container_add (GTK_CONTAINER (page->cuisines_box2), tile);
         }
 }
 
@@ -207,13 +288,17 @@ gr_cuisines_page_class_init (GrCuisinesPageClass *klass)
 
         gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Recipes/gr-cuisines-page.ui");
 
-        gtk_widget_class_bind_template_child (widget_class, GrCuisinesPage, top_box);
+        gtk_widget_class_bind_template_child (widget_class, GrCuisinesPage, cuisines_box);
+        gtk_widget_class_bind_template_child (widget_class, GrCuisinesPage, cuisines_box2);
+        gtk_widget_class_bind_template_child (widget_class, GrCuisinesPage, cuisines_more);
+        gtk_widget_class_bind_template_child (widget_class, GrCuisinesPage, cuisines_expander_image);
         gtk_widget_class_bind_template_child (widget_class, GrCuisinesPage, seasonal_box);
         gtk_widget_class_bind_template_child (widget_class, GrCuisinesPage, seasonal_box2);
         gtk_widget_class_bind_template_child (widget_class, GrCuisinesPage, seasonal_more);
         gtk_widget_class_bind_template_child (widget_class, GrCuisinesPage, seasonal_expander_image);
 
-         gtk_widget_class_bind_template_callback (widget_class, expander_button_clicked);
+         gtk_widget_class_bind_template_callback (widget_class, season_expander_button_clicked);
+         gtk_widget_class_bind_template_callback (widget_class, cuisines_expander_button_clicked);
 }
 
 GtkWidget *
@@ -250,4 +335,5 @@ connect_store_signals (GrCuisinesPage *page)
         g_signal_connect_swapped (store, "recipe-added", G_CALLBACK (cuisines_page_reload), page);
         g_signal_connect_swapped (store, "recipe-removed", G_CALLBACK (cuisines_page_reload), page);
         g_signal_connect_swapped (store, "recipe-changed", G_CALLBACK (cuisines_page_reload), page);
+        g_signal_connect_swapped (store, "reloaded", G_CALLBACK (gr_cuisines_page_refresh), page);
 }
