@@ -31,10 +31,12 @@
 #include "gr-recipe-formatter.h"
 #include "gr-image.h"
 #include "gr-chef.h"
+#include "gr-app.h"
 #include "gr-recipe.h"
 #include "gr-recipe-store.h"
 #include "gr-utils.h"
 #include "gr-mail.h"
+#include "gr-recipe-printer.h"
 
 
 struct _GrRecipeExporter
@@ -50,6 +52,7 @@ struct _GrRecipeExporter
 #endif
         GFile *output; /* the location where compressor writes the archive */
         GList *sources; /* the list of all files */
+        GList *pdf_sources;
         char *dir;
 
         gboolean just_export;
@@ -70,6 +73,7 @@ gr_recipe_exporter_finalize (GObject *object)
         g_list_free_full (exporter->recipes, g_object_unref);
         g_clear_object (&exporter->output);
         g_list_free_full (exporter->sources, g_object_unref);
+        g_list_free_full (exporter->pdf_sources, g_object_unref);
         g_free (exporter->dir);
 
         G_OBJECT_CLASS (gr_recipe_exporter_parent_class)->finalize (object);
@@ -122,7 +126,9 @@ cleanup_export (GrRecipeExporter *exporter)
         exporter->recipes = NULL;
         g_clear_object (&exporter->output);
         g_list_free_full (exporter->sources, g_object_unref);
+        g_list_free_full (exporter->pdf_sources, g_object_unref);
         exporter->sources = NULL;
+        exporter->pdf_sources = NULL;
 
         gr_recipe_store_clear_export_list (gr_recipe_store_get ());
 }
@@ -183,7 +189,11 @@ completed_cb (AutoarCompressor *compressor,
         const char *address;
         const char *subject;
         g_autofree char *body = NULL;
-        const char *attachments[2];
+        g_auto(GStrv) attachments;
+        int i = 1;
+        GList *tmp_pdf_list;
+        int pdf_sources_length = g_list_length (exporter->pdf_sources);
+        attachments = g_new (char*, pdf_sources_length + 2);
 
         if (exporter->just_export) {
                 g_signal_emit (exporter, done_signal, 0, exporter->output);
@@ -227,11 +237,16 @@ completed_cb (AutoarCompressor *compressor,
 
         path = g_file_get_path (exporter->output);
 
-        attachments[0] = path;
-        attachments[1] = NULL;
+        attachments[0] = g_strdup (path);
+
+        for (tmp_pdf_list = exporter->pdf_sources; i-1 < pdf_sources_length; i++, tmp_pdf_list = tmp_pdf_list->next)
+                attachments[i] = g_file_get_path (tmp_pdf_list->data);
+
+        attachments[pdf_sources_length + 1] = NULL;
+
 
         gr_send_mail (GTK_WINDOW (exporter->window),
-                      address, subject, body, attachments,
+                      address, subject, body, (const char **)attachments,
                       mail_done, exporter);
 }
 #endif
@@ -268,6 +283,9 @@ export_one_recipe (GrRecipeExporter  *exporter,
         int i;
         g_autofree char *imagedir = NULL;
 
+        GrRecipePrinter *printer;
+        GFile *recipe_pdf;
+        printer = gr_recipe_printer_new (exporter->window);
         key = gr_recipe_get_id (recipe);
         name = gr_recipe_get_name (recipe);
         author = gr_recipe_get_author (recipe);
@@ -295,12 +313,16 @@ export_one_recipe (GrRecipeExporter  *exporter,
         paths = g_new0 (char *, images->len + 1);
         for (i = 0; i < images->len; i++) {
                 GrImage *ri = g_ptr_array_index (images, i);
+                g_autoptr(GdkPixbuf) pixbuf = NULL;
                 g_autoptr(GFile) source = NULL;
                 g_autoptr(GFile) dest = NULL;
+                g_autofree char *path = NULL;
                 g_autofree char *basename = NULL;
                 g_autofree char *destname = NULL;
 
-                source = g_file_new_for_path (gr_image_get_path (ri));
+                pixbuf = gr_image_load_sync (ri, 400, 400, TRUE);
+                path = gr_image_get_cache_path (ri);
+                source = g_file_new_for_path (path);
                 basename = g_file_get_basename (source);
                 destname = g_build_filename (imagedir, basename, NULL);
 
@@ -331,6 +353,9 @@ export_one_recipe (GrRecipeExporter  *exporter,
 
         g_key_file_set_string_list (keyfile, key, "Images", (const char * const *)paths, g_strv_length (paths));
 
+        recipe_pdf = gr_recipe_printer_get_pdf (printer, recipe);
+        exporter->pdf_sources = g_list_append (exporter->pdf_sources, recipe_pdf);
+
         if (ctime) {
                 g_autofree char *created = date_time_to_string (ctime);
                 g_key_file_set_string (keyfile, key, "Created", created);
@@ -357,6 +382,7 @@ export_one_chef (GrRecipeExporter  *exporter,
         const char *fullname;
         const char *description;
         const char *image_path;
+        g_autoptr(GMainLoop) loop = NULL;
 
         key = gr_chef_get_id (chef);
         name = gr_chef_get_name (chef);
@@ -366,11 +392,18 @@ export_one_chef (GrRecipeExporter  *exporter,
         if (image_path && image_path[0]) {
                 g_autoptr(GFile) source = NULL;
                 g_autoptr(GFile) dest = NULL;
+                g_autoptr(GrImage) ri = NULL;
+                g_autoptr(GdkPixbuf) pixbuf = NULL;
                 g_autofree char *basename = NULL;
                 g_autofree char *destname = NULL;
                 g_autofree char *path = NULL;
+                g_autofree char *cache_path = NULL;
 
-                source = g_file_new_for_path (image_path);
+                ri = gr_image_new (gr_app_get_soup_session (GR_APP (g_application_get_default ())), gr_chef_get_id (chef), image_path);
+                pixbuf = gr_image_load_sync (ri, 400, 400, FALSE);
+
+                cache_path = gr_image_get_cache_path (ri);
+                source = g_file_new_for_path (cache_path);
                 basename = g_file_get_basename (source);
                 path = g_build_filename ("images", basename, NULL);
                 destname = g_build_filename (exporter->dir, path, NULL);
@@ -411,6 +444,7 @@ prepare_export (GrRecipeExporter  *exporter,
 
         g_assert (exporter->dir == NULL);
         g_assert (exporter->sources == NULL);
+        g_assert (exporter->pdf_sources == NULL);
 
         exporter->dir = g_mkdtemp (g_build_filename (g_get_tmp_dir (), "recipeXXXXXX", NULL));
 
@@ -752,6 +786,7 @@ gr_recipe_exporter_export (GrRecipeExporter *exporter,
         keys = gr_recipe_store_get_export_list (store);
 
         g_list_free_full (exporter->recipes, g_object_unref);
+        g_list_free_full (exporter->pdf_sources, g_object_unref);
         exporter->recipes = NULL;
 
         for (i = 0; keys[i]; i++) {
